from __future__ import absolute_import
import requests
from .models import ModelList
from dateutil.tz import tzlocal
import datetime
import json


class Client(object):
    def __init__(self, token=None, url=None):
        import ureactme
        if token is not None:
            ureactme.api_key = token

        if url is not None:
            ureactme.url = url

        self.token = token or ureactme.api_key
        self.url = url or ureactme.url

    def get_headers(self):
        return {"Content-Type": "application/json",
                "Authorization": "Token %s" % self.token}

    def post(self, url, data):
        requests.post(url, data, headers=self.get_headers())

    def send_event(self, user, category, action, label=None,
                   value=None, data=None, date=None):
        from ureactme.models import Event
        now = datetime.datetime.now(tzlocal())
        payload = {"date": date or now.isoformat(),
                   "category": category,
                   "action": action,
                   "label": label,
                   "user": user,
                   "value": value,
                   "data": data}
        url = self.url + Event.API_ENDPOINT
        r = requests.post(url, json.dumps(payload), headers=self.get_headers())
        if r.status_code > 299 or r.status_code < 200:
            raise ValueError(r.content)

    def get_statistics(self, day_range, category=None, action=None, label=None,
                       user=None, fields=None):
        from .models import DayStatistic
        from_day, to_day = day_range
        url = (self.url + DayStatistic.API_ENDPOINT)
        params = {"from_day": from_day, "to_day": to_day}
        if category is not None:
            params['metric_category'] = category
        if action is not None:
            params['metric_action'] = action
        if label is not None:
            params['metric_label'] = label
        if user:
            params['user'] = user
        if fields:
            params['fields'] = ','.join(fields)
        r = requests.get(url, params=params, headers=self.get_headers())
        if r.status_code > 299 or r.status_code < 200:
            raise ValueError(r.content)
        data = r.json()

        return ModelList(self, params, data, DayStatistic)

    def get_object_list(self, modelcls, url=None, **filters):
        """
        Returns the list of objects using the given **kwargs as filters

        If url is not none, use this URL when fetching, instead
        of modelcls.API_ENDPOINT variable
        """
        if url is None:
            url = self.url + modelcls.API_ENDPOINT
        r = requests.get(url, params=filters, headers=self.get_headers())
        if r.status_code > 299 or r.status_code < 200:
            raise ValueError(r.content)
        data = r.json()

        return ModelList(self, filters, data, modelcls)



from __future__ import absolute_import
from dateutil.parser import parse as parse_datetime


class Model(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    @classmethod
    def get_by_id(cls, id):
        from . import client
        return list(client.Client().get_object_list(cls, id__eq=id))[0]

    @classmethod
    def get_list(cls, **filter):
        from . import client
        return client.Client().get_object_list(cls, **filter)


class ModelList(object):
    def __init__(self, client, filters, payload, modelcls):
        self.client = client
        self.filters = filters
        self.meta = payload['meta']
        self.objects = payload['data']
        self.modelcls = modelcls

    def __len__(self):
        return self.meta['total']

    def __iter__(self):
        i = 0
        while True:
            if i < len(self.objects):
                yield self.modelcls(**self.objects[i])
                i += 1
            elif self.meta['page'] < self.meta['total_pages']:
                # get next page
                filters = self.filters.copy()
                filters['page'] = self.meta['page'] + 1
                for k in self.client.get_object_list(self.modelcls, **filters):
                    yield k
                break
            else:
                break


class User(Model):
    API_ENDPOINT = "/api/v2/user"

    def get_events(self, metric, date):
        from . import client
        metric_id = metric.id if isinstance(metric, Metric) else metric
        filters = {"metric": metric_id, "user": self.id, "day": date}
        return client.Client().get_object_list(Event, **filters)

    def __str__(self):
        return self.id


class Metric(Model):
    API_ENDPOINT = "/api/v2/metric"

    def __str__(self):
        return self.id


class Event(Model):
    API_ENDPOINT = "/api/v2/event"

    def __init__(self, **kwargs):
        super(Event, self).__init__(**kwargs)
        if isinstance(self.created_at, basestring):
            self.created_at = parse_datetime(self.created_at)

    def __str__(self):
        return "%s (%s; %s)" % (self.created_at, self.value, self.data)


class DayStatistic(Model):
    API_ENDPOINT = '/api/v2/stats'



from setuptools import setup, Command


class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import unittest

        modules = ['ureactme.tests.test_client',
                   'ureactme.tests.test_models']
        suite = unittest.TestSuite()
        suite.addTests(unittest.defaultTestLoader.loadTestsFromNames(modules))
        unittest.TextTestRunner().run(suite)

long_description = "Client library for ureact.me API service"

try:
    long_description = open("README.md").read()
except:
    pass

setup(name='ureactme',
      version='0.2.7',
      packages=['ureactme', ],
      license='MIT',
      author='Thiago F. Pappacena',
      author_email='pappacena@gmail.com',
      url='https://github.com/pappacena/',
      description='UReact.me API lib',
      long_description=long_description,
      install_requires=['setuptools>=17.1', 'requests', 'python-dateutil',
                        'mock', 'freezegun>=0.3.5'],
      tests_require=['freezegun>=0.3.5'],
      cmdclass={'test': TestCommand},
      classifiers=['Environment :: Web Environment',
                   'Intended Audience :: Developers',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3',
                   'Programming Language :: Python :: 3.2',
                   'Programming Language :: Python :: 3.3',
                   'Programming Language :: Python :: 3.4'])
